==> filter-dnsbl-0.2.tar.gz: filter-dnsbl-0.2/filter-dnsbl.go <==
// $Id: filter-dnsbl.go 13 2019-01-01 22:20:17Z umaxx $
// Copyright (c) 2018 Joerg Jung <mail@umaxx.net>
//
// Permission to use, copy, modify, and distribute this software for any
// purpose with or without fee is hereby granted, provided that the above
// copyright notice and this permission notice appear in all copies.
//
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
// WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
// ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
// ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
// OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

// filter-dnsbl - opensmtpd filter for DNSBL/DNSWL

package main

import (
	"bufio"
	"fmt"
	"log"
	"log/syslog"
	"net"
	"os"
	"strings"
)

const (
	v  = "0.2"
	yr = 2018
)

var (
	bl = []string{"bl.spamcop.net", "zen.spamhaus.org"}
	wl = []string{"list.dnswl.org"}
	l3 *syslog.Writer
)

func reverse(ip net.IP) string {
	rip := strings.Split(ip.String(), ".")
	for i, j := 0, len(rip)-1; i < len(rip)/2; i, j = i+1, j-1 {
		rip[i], rip[j] = rip[j], rip[i]
	}
	return strings.Join(rip, ".")
}

func lookup(ip net.IP, l string) bool {
	r, e := net.LookupHost(fmt.Sprintf("%s.%s", reverse(ip), l))
	if e != nil {
		l3.Debug(fmt.Sprintln(e))
		return false
	}
	return (len(r) > 0)
}

func check(tok string, sid string, ip net.IP) {
	if ip.To4() == nil {
		l3.Debug(fmt.Sprintln("non-ipv4 address", ip))
		goto proceed
	}
	for _, l := range wl {
		if lookup(ip, l) {
			l3.Debug(fmt.Sprintln("white listed", ip, l))
			goto proceed
		}
	}
	for _, l := range bl {
		if lookup(ip, l) {
			l3.Debug(fmt.Sprintln("black listed", ip, l))
			fmt.Printf("filter-result|%s|%s|reject|554 5.7.1 Address in DNSBL\n", tok, sid)
			return
		}
	}
proceed:
	fmt.Printf("filter-result|%s|%s|proceed\n", tok, sid)
}

func register() {
	l3.Info("register")
	fmt.Println("register|filter|smtp-in|connect")
	fmt.Println("register|ready")
}

func run() {
	l3.Info("start")
	defer l3.Info("exit")
	in := bufio.NewScanner(os.Stdin)
	for in.Scan() {
		f := strings.Split(in.Text(), "|")
		t, ver, ev, tok, sid, ip := f[0], f[1], f[4], f[5], f[6], net.ParseIP(f[8])
		if t != "filter" || ver != "1" || ev != "connect" || ip == nil {
			l3.Err(fmt.Sprintln("protocol", t, ver, ev, ip))
			return
		}
		go check(tok, sid, ip)
	}
	if e := in.Err(); e != nil {
		l3.Err(fmt.Sprintln("scanner", e))
		return
	}
}

func init() {
	log.SetFlags(log.Lshortfile)
}

func main() {
	var e error
	if len(os.Args) == 2 && os.Args[1] == "version" {
		fmt.Println("filter-dnsbl", v, "(c)", yr, "Joerg Jung")
		return
	}
	if len(os.Args) != 1 {
		log.Fatalf("usage: filter-dnsbl\n%27sfilter-dnsbl version\n", "")
	}
	if l3, e = syslog.New(syslog.LOG_MAIL, "filter-dnsbl"); e != nil { // todo: LOG_PERROR+LOG_PID?
		log.Fatal(e)
	}
	defer l3.Close()
	register()
	run()
}

==> filter-dnsbl-0.3.tar.gz: filter-dnsbl-0.3/filter-dnsbl.go <==
// $Id: filter-dnsbl.go 17 2019-02-10 19:51:30Z umaxx $
// Copyright (c) 2018-2019 Joerg Jung <mail@umaxx.net>
//
// Permission to use, copy, modify, and distribute this software for any
// purpose with or without fee is hereby granted, provided that the above
// copyright notice and this permission notice appear in all copies.
//
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
// WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
// ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
// ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
// OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

// filter-dnsbl - opensmtpd filter for DNSBL/DNSWL

package main

import (
	"bufio"
	"fmt"
	"log"
	"log/syslog"
	"net"
	"os"
	"strings"
)

const (
	v  = "0.3"
	yr = "2018-2019"
)

var (
	bl = "zen.spamhaus.org"
	wl = "list.dnswl.org"
	l3 *syslog.Writer
)

func reverse(ip net.IP) string {
	rip := strings.Split(ip.String(), ".")
	for i, j := 0, len(rip)-1; i < len(rip)/2; i, j = i+1, j-1 {
		rip[i], rip[j] = rip[j], rip[i]
	}
	return strings.Join(rip, ".")
}

func lookup(ip net.IP, l string) bool {
	r, e := net.LookupHost(fmt.Sprintf("%s.%s", reverse(ip), l))
	if e != nil {
		l3.Debug(fmt.Sprintln(e))
		return false
	}
	return (len(r) > 0)
}

func check(tok string, sid string, ip net.IP) {
	if ip.To4() == nil { // todo: handle ipv6
		l3.Debug(fmt.Sprintln("non-ipv4 address", ip))
		goto proceed
	}
	if lookup(ip, wl) {
		l3.Debug(fmt.Sprintln("white listed", ip, wl))
		goto proceed
	}
	if lookup(ip, bl) {
		l3.Debug(fmt.Sprintln("black listed", ip, bl))
		fmt.Printf("filter-result|%s|%s|reject|554 5.7.1 Address in DNSBL\n", tok, sid)
		return
	}
proceed:
	fmt.Printf("filter-result|%s|%s|proceed\n", tok, sid)
}

func register() {
	l3.Info("register")
	fmt.Println("register|filter|smtp-in|connect")
	fmt.Println("register|ready")
}

func run() {
	l3.Info("start")
	defer l3.Info("exit")
	in := bufio.NewScanner(os.Stdin)
	for in.Scan() {
		f := strings.Split(in.Text(), "|")
		t, ver, ev, sid, tok, ip := f[0], f[1], f[4], f[5], f[6], net.ParseIP(f[8])
		if t != "filter" || ver != "1" || ev != "connect" || ip == nil {
			l3.Err(fmt.Sprintln("protocol", t, ver, ev, ip))
			return
		}
		go check(tok, sid, ip)
	}
	if e := in.Err(); e != nil {
		l3.Err(fmt.Sprintln("scanner", e))
		return
	}
}

func init() {
	log.SetFlags(log.Lshortfile)
}

func main() {
	var e error
	if len(os.Args) == 2 && os.Args[1] == "version" {
		fmt.Println("filter-dnsbl", v, "(c)", yr, "Joerg Jung")
		return
	}
	if len(os.Args) > 3 {
		log.Fatalf("usage: filter-dnsbl [<blacklist>] [<whitelist>]\n%27sfilter-dnsbl version\n", "")
	}
	if len(os.Args) >= 2 {
		bl = os.Args[1]
	}
	if len(os.Args) == 3 {
		wl = os.Args[2]
	}
	if l3, e = syslog.New(syslog.LOG_MAIL, "filter-dnsbl"); e != nil {
		log.Fatal(e)
	}
	defer l3.Close()
	register()
	run()
}
